@@ -158,7 +158,7 @@ fs_rtp_tfrc_init (FsRtpTfrc *self)
 
   self->extension_type = EXTENSION_NONE;
   self->extension_id = 0;
-  memset (self->pts, 0, 128);
+  memset (self->pts, 0, 128 * sizeof (gboolean));
 
   self->systemclock = gst_system_clock_obtain ();
 }
@@ -1354,9 +1354,11 @@ fs_rtp_tfrc_new (FsRtpSession *fsrtpsession)
   gst_object_unref (rtpmuxer);
 
   self->in_rtp_probe_id = gst_pad_add_probe (self->in_rtp_pad,
-      GST_PAD_PROBE_TYPE_BUFFER, incoming_rtp_probe, self, NULL);
+      GST_PAD_PROBE_TYPE_BUFFER, incoming_rtp_probe,
+      g_object_ref (self), (GDestroyNotify) g_object_unref);
   self->in_rtcp_probe_id = gst_pad_add_probe (self->in_rtcp_pad,
-      GST_PAD_PROBE_TYPE_BUFFER, incoming_rtcp_probe, self, NULL);
+      GST_PAD_PROBE_TYPE_BUFFER, incoming_rtcp_probe,
+      g_object_ref (self), (GDestroyNotify) g_object_unref);
 
 
   self->on_ssrc_validated_id = g_signal_connect_object (self->rtpsession,
@@ -1444,7 +1446,7 @@ fs_rtp_tfrc_codecs_updated (FsRtpTfrc *self,
 
   GST_OBJECT_LOCK (self);
 
-  memset (self->pts, 0, 128);
+  memset (self->pts, 0, 128 * sizeof (gboolean));
   for (item = codec_associations; item; item = item->next)
   {
     CodecAssociation *ca = item->data;
