@@ -166,7 +166,7 @@ fs_multicast_stream_transmitter_get_type (void)
 }
 
 GType
-fs_multicast_stream_transmitter_register_type (FsPlugin *module)
+fs_multicast_stream_transmitter_register_type (FsPlugin *module G_GNUC_UNUSED)
 {
   static const GTypeInfo info = {
     sizeof (FsMulticastStreamTransmitterClass),
@@ -180,8 +180,8 @@ fs_multicast_stream_transmitter_register_type (FsPlugin *module)
     (GInstanceInitFunc) fs_multicast_stream_transmitter_init
   };
 
-  type = g_type_module_register_type (G_TYPE_MODULE (module),
-    FS_TYPE_STREAM_TRANSMITTER, "FsMulticastStreamTransmitter", &info, 0);
+  type = g_type_register_static (FS_TYPE_STREAM_TRANSMITTER,
+      "FsMulticastStreamTransmitter", &info, 0);
 
   return type;
 }
